@@ -1,5 +1,5 @@
 /* delkey.c - delete keys
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -43,9 +43,11 @@
 
 /****************
  * Delete a public or secret key from a keyring.
+ * r_sec_avail will be set if a secret key is available and the public
+ * key can't be deleted for that reason.
  */
-int
-delete_key( const char *username, int secret )
+static int
+do_delete_key( const char *username, int secret, int *r_sec_avail )
 {
     int rc = 0;
     KBNODE keyblock = NULL;
@@ -57,11 +59,13 @@ delete_key( const char *username, int secret )
     int okay=0;
     int yes;
 
+    *r_sec_avail = 0;
     /* search the userid */
     rc = secret? find_secret_keyblock_byname( &kbpos, username )
 	       : find_keyblock_byname( &kbpos, username );
     if( rc ) {
 	log_error(_("%s: user not found\n"), username );
+	write_status_text( STATUS_DELETE_PROBLEM, "1" );
 	goto leave;
     }
 
@@ -89,14 +93,13 @@ delete_key( const char *username, int secret )
 	keyid_from_pk( pk, keyid );
 	rc = seckey_available( keyid );
 	if( !rc ) {
-	    log_error(_(
-	    "there is a secret key for this public key!\n"));
-	    log_info(_(
-	    "use option \"--delete-secret-key\" to delete it first.\n"));
-	    rc = -1;
+            *r_sec_avail = 1;
+            rc = -1;
+            goto leave;
 	}
-	else if( rc != G10ERR_NO_SECKEY )
+	else if( rc != G10ERR_NO_SECKEY ) {
 	    log_error("%s: get secret key: %s\n", username, g10_errstr(rc) );
+	}
 	else
 	    rc = 0;
     }
@@ -124,7 +127,7 @@ delete_key( const char *username, int secret )
 		      pubkey_letter( pk->pubkey_algo ),
 		      keyid[1], datestr_from_pk(pk) );
 	p = get_user_id( keyid, &n );
-	tty_print_string( p, n );
+	tty_print_utf8_string( p, n );
 	m_free(p);
 	tty_printf("\n\n");
 
@@ -158,3 +161,28 @@ delete_key( const char *username, int secret )
     return rc;
 }
 
+/****************
+ * Delete a public or secret key from a keyring.
+ */
+int
+delete_key( const char *username, int secret, int allow_both )
+{
+    int rc, avail;
+
+    rc = do_delete_key (username, secret, &avail );
+    if ( rc && avail ) { 
+        if ( allow_both ) {
+            rc = do_delete_key (username, 1, &avail );
+            if ( !rc )
+                rc = do_delete_key (username, 0, &avail );
+        }
+        else {
+            log_error(_(
+                "there is a secret key for this public key!\n"));
+            log_info(_(
+                "use option \"--delete-secret-key\" to delete it first.\n"));
+            write_status_text( STATUS_DELETE_PROBLEM, "2" );
+        }
+    }
+    return rc;
+}
