@@ -1,5 +1,5 @@
 /* ttyio.c -  tty i/O functions
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -50,7 +50,11 @@
 #include "ttyio.h"
 
 #define CONTROL_D ('D' - 'A' + 1)
-
+#ifdef __VMS
+  #define TERMDEVICE "/dev/tty"
+#else
+  #define TERMDEVICE "/dev/tty"
+#endif
 
 #ifdef __MINGW32__ /* use the odd Win32 functions */
 static struct {
@@ -120,7 +124,7 @@ init_ttyfp(void)
   #elif defined(__EMX__)
     ttyfp = stdout; /* Fixme: replace by the real functions: see wklib */
   #else
-    ttyfp = batchmode? stderr : fopen("/dev/tty", "r+");
+    ttyfp = batchmode? stderr : fopen(TERMDEVICE, "r+");
     if( !ttyfp ) {
 	log_error("cannot open /dev/tty: %s\n", strerror(errno) );
 	exit(2);
@@ -193,7 +197,7 @@ tty_printf( const char *fmt, ... )
 	if( !WriteConsoleA( con.out, buf, n, &nwritten, NULL ) )
 	    log_fatal("WriteConsole failed: rc=%d", (int)GetLastError() );
 	if( n != nwritten )
-	    log_fatal("WriteConsole failed: %d != %ld\n", n, nwritten );
+	    log_fatal("WriteConsole failed: %d != %d\n", n, nwritten );
 	last_prompt_len += n;
     }
   #else
